package org.magmafoundation.magma;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.Arrays;
import java.util.List;

public class MagmaModLoader {

    private static final List<String> dontCheck = Arrays.asList(
            "com/destroystokyo/paper/",
            "com/mojang/",
            "io/papermc/paper/",
            "net/minecraft/",
            "net/kyori/adventure/bossbar/",
            "net/minecraftforge/",
            "org/magmafoundation/magma/",
            "org/bukkit/",
            "org/spigotmc/",
            "org/spongepowered/",
            "META-INF/",
            "assets/",
            "*.json",
            "*.png",
            "*.yml"
    );

    private static final Logger LOGGER = LogManager.getLogger("MagmaModLoader");

    public static boolean isPackageExcluded(String mod, String resourcePath) {
        if (mod.startsWith("forge-" + MagmaConstants.FORGE_VERSION))
            return false;

        for (String resource : dontCheck) {
            if (resource.startsWith("*") && resourcePath.contains(resource.substring(1)))
                return false;
            if (resourcePath.startsWith(resource))
                return false;
        }

        for (MOD value : MOD.values()) {
            if (mod.startsWith(value.name()) && value.isPackageExcluded(resourcePath)) {
                LOGGER.debug("Excluding {} from {}", resourcePath, mod);
                return true;
            }
        }

        return false;
    }

    private enum MOD {
        OpenTerrainGenerator("org/yaml/snakeyaml/");

        private final  List<String> exclusions;

        MOD(String... exclusions) {
            this.exclusions = Arrays.asList(exclusions);
        }

        boolean isPackageExcluded(String packageName) {
            return exclusions.stream().anyMatch(packageName::startsWith);
        }
    }
}
