/*
 * Magma Server
 * Copyright (C) 2019-2022.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package org.magmafoundation.magma;

import java.io.*;
import java.lang.reflect.Array;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Map;

import org.magmafoundation.magma.LibraryDownloader.LibraryDownloader;
import org.magmafoundation.magma.LibraryDownloader.lib.Library;
import org.magmafoundation.magma.betterui.BetterUI;
import org.magmafoundation.magma.libs.MagmaInstaller;
import org.magmafoundation.magma.updater.MagmaUpdater;
import org.magmafoundation.magma.utils.JarLoader;
import org.magmafoundation.magma.utils.JarUtils;
import org.yaml.snakeyaml.Yaml;

import static org.magmafoundation.magma.MagmaConstants.*;

/**
 * MagmaStart
 *
 * @author Hexeption admin@hexeption.co.uk / Malcolm
 * @since 24/05/2022 - 7:21 pm
 */
public class MagmaStart {

    public static boolean enableUpdate = true;

    public static void main(String[] args) throws Exception {
        BetterUI.printTitle(NAME, BRAND, System.getProperty("java.version") + " (" + System.getProperty("java.vendor") + ")", VERSION, BUKKIT_VERSION, FORGE_VERSION);

        if (!BetterUI.checkEula(Paths.get("eula.txt")))
            System.exit(0);

        File libraries_folder = new File(String.format("%s/%s/", JarUtils.getJarDir(), MagmaConstants.INSTALLER_LIBRARIES_FOLDER));
        LibraryDownloader libraryDownloader = new LibraryDownloader(libraries_folder, new InputStreamReader(MagmaStart.class.getResourceAsStream("/magma_libs.json")));

        libraryDownloader.getServerJar(
                "https://launcher.mojang.com/v1/objects/1b557e7b033b583cd9f66746b7a9ab1ec1673ced/server.jar",
                "minecraft_server.1.16.5",
                "c10b74188efc4ed6960db49c9ade50ce"
        );

        libraryDownloader.start();

        for (Library library : libraryDownloader.getLibraries()) {
            JarLoader.loadJar(new File(libraries_folder, library.getFullPath()));
        }

        MagmaInstaller.install();
        JarLoader.loadJar(MagmaInstaller.getExtras());

        //The server can be run with Java 16+
        if(Float.parseFloat(System.getProperty("java.class.version")) >= 60.0) {
            Class.forName("org.magmafoundation.magma.util.MagmaModuleManager", false, URLClassLoader.newInstance(new URL[]{MagmaInstaller.universalJar.toURI().toURL()})).getDeclaredConstructor().newInstance();
        }

        Path path = Paths.get("magma.yml");
        if (Files.exists(path)) {
            try (InputStream stream = Files.newInputStream(path)) {
                Yaml yaml = new Yaml();
                Map<String, Object> data = yaml.load(stream);
                Map<String, Object> forge = (Map<String, Object>) data.get("magma");
                MagmaUpdater updater = new MagmaUpdater();
                if (enableUpdate) {
                    System.out.println("Checking for updates...");
                    if (updater.versionChecker() && forge.get("auto-update").equals(true))
                        updater.downloadJar();
                }
            }
        }
    }

    public static String[] remove(String[] array, String element) {
        if (array.length > 0) {
            int index = -1;
            for (int i = 0; i < array.length; i++) {
                if (array[i].equalsIgnoreCase(element)) {
                    index = i;
                    break;
                }
            }
            if (index >= 0) {
                String[] copy = (String[]) Array.newInstance(array.getClass()
                        .getComponentType(), array.length - 1);
                if (copy.length > 0) {
                    System.arraycopy(array, 0, copy, 0, index);
                    System.arraycopy(array, index + 1, copy, index, copy.length - index);
                }
                return copy;
            }
        }
        return array;
    }
}
