package com.destroystokyo.paper;

import static com.destroystokyo.paper.PaperConfigNR.log;

import java.util.List;

import org.bukkit.configuration.file.YamlConfiguration;
import org.spigotmc.SpigotWorldConfig;

public class PaperWorldConfig {

	private final String worldName;
	private final SpigotWorldConfig spigotConfig;
	private YamlConfiguration config;
	private boolean verbose;

	public PaperWorldConfig(String worldName, SpigotWorldConfig spigotConfig) {
		this.worldName = worldName;
		this.spigotConfig = spigotConfig;
		this.config = PaperConfigNR.config;
		init();
	}

	public void init() {
		this.config = PaperConfigNR.config; // grab updated reference
		log("-------- World Settings For [" + worldName + "] --------");
		PaperConfigNR.readConfig(PaperWorldConfig.class, this);
	}

	private void set(String path, Object val) {
		config.set("world-settings.default." + path, val);
		if (config.get("world-settings." + worldName + "." + path) != null) {
			config.set("world-settings." + worldName + "." + path, val);
		}
	}

	private boolean getBoolean(String path, boolean def) {
		config.addDefault("world-settings.default." + path, def);
		return config.getBoolean("world-settings." + worldName + "." + path, config.getBoolean("world-settings.default." + path));
	}

	private double getDouble(String path, double def) {
		config.addDefault("world-settings.default." + path, def);
		return config.getDouble("world-settings." + worldName + "." + path, config.getDouble("world-settings.default." + path));
	}

	private int getInt(String path, int def) {
		config.addDefault("world-settings.default." + path, def);
		return config.getInt("world-settings." + worldName + "." + path, config.getInt("world-settings.default." + path));
	}

	private float getFloat(String path, float def) {
		// TODO: Figure out why getFloat() always returns the default value.
		return (float) getDouble(path, (double) def);
	}

	private <T> List<T> getList(String path, List<T> def) {
		config.addDefault("world-settings.default." + path, def);
		return (List<T>) config.getList("world-settings." + worldName + "." + path, config.getList("world-settings.default." + path));
	}

	private String getString(String path, String def) {
		config.addDefault("world-settings.default." + path, def);
		return config.getString("world-settings." + worldName + "." + path, config.getString("world-settings.default." + path));
	}
}
